require_relative 'project'
require_relative 'die'
require_relative 'funding_round'
require_relative 'pledge_pool'
require 'csv'

module Crowdfund
  class GroupFundRequest
    attr_reader :projects

    def initialize(name)
      @name = name
      @projects = []
    end

    def add_project(project)
      @projects << project
    end

    def load_projects(from_file)
      CSV.foreach(from_file) do |row|
        project = Project.new(row[0],Integer(row[1]),Integer(row[2]))
        add_project(project)
      end
    end

    def request_funding(rounds)
      puts "\nThere are #{@projects.count} projects in #{@name}:"
      puts @projects
      puts "\nThere are #{PledgePool::PLEDGES.count} possible pledge amounts:"
      PledgePool::PLEDGES.each { |pledge| puts "A #{pledge.name} pledge is worth $#{pledge.amount}" }
      n = 0
      1.upto(rounds) do
        n += 1
        puts "\n  Round #{n}:"
        @projects.each { |project| puts "The funding goal of #{project.name} is $#{project.goal}." }
        @projects.each { |project|
          FundingRound.do_round(project)
          if project.fully_funded?
            puts "#{project.name} is FULLY FUNDED. (Currently $#{project.funding} toward the goal of $#{project.goal}.)\n"
          else
            puts "#{project.name} still needs funds. (Currently $#{project.funding} toward the goal of $#{project.goal}.)\n"
          end
        }
      end
    end

    def print_funding_status
      puts "\nCurrent Funding of All Projects: "
      @projects.sort.each { |project| puts "#{project.name} funding: $#{project.funding} of $#{project.goal} "}
      fully_funded, underfunded = @projects.partition { |project| project.fully_funded? }
      puts "\nFully Funded Projects: #{fully_funded.size}\nUnderfunded Projects: #{underfunded.size}"
    end

    def underfunded
      @projects.sort.reject { |project| project.fully_funded? }
    end

    def print_underfunded
      underfunded.each { |project| puts "#{project.name} still needs $#{project.funds_needed}."}
    end

    def save_underfunded
      to_file = File.join(File.dirname(File.dirname(File.dirname(__FILE__))), "underfunded.txt")
      File.open(to_file, "w") do |file|
        file.puts "Projects that still need funds:"
        underfunded.each do |project|
          file.puts "#{project.name} still needs $#{project.funds_needed}."
        end
      end
      puts "The list of underfunded projects has been saved in \"underfunded.txt\"."
    end
  end
end

if __FILE__ == $0
  project1 = Project.new("Test1", 10_000)
  project2 = Project.new("Test2", 20_000, 2_000)
  group = GroupFundRequest.new("Test Group")
  group.add_project(project1)
  group.add_project(project2)
  group.request_funding(2)
  project3 = Project.new("Test3", 1_000)
  group.add_project(project3)
  group.request_funding(3)
  group.print_funding_status
  group.print_underfunded
end
