require_relative 'pledge_pool'
require_relative 'fundable'

module Crowdfund
  class Project
    include Fundable

    attr_reader :goal, :pledges
    attr_accessor :name, :funding

    def initialize (name, goal, funding=0)
      @name = name
      @goal = goal
      @funding = funding
      @pledges = Hash.new(0)
    end

    def <=>(other_project)
      other_project.funding <=> funding
    end


    def receive_pledge(match_factor=1)
      pledge = PledgePool.random
      puts "#{name} received a #{pledge.name} pledge worth $#{pledge.amount}."
      @pledges[pledge.name] += pledge.amount * match_factor
      @funding += pledge.amount * match_factor
      puts "#{name}'s pledge amounts:"
      each_pledge { |pledge| puts "$#{pledge.amount} in #{pledge.name} pledges" }
    end

    def each_pledge
      @pledges.sort_by { |name, amount| amount }.each { |name, amount| yield Pledge.new(name, amount) }
    end

    def to_s
      "#{@name} has $#{@funding} in funding towards a goal of $#{@goal}."
    end
  end
end

if __FILE__ == $0
  project = Project.new("Test Project", 10_000)
  puts project.name
  puts project.funding
  project.add_funds
  puts project.funding
  project.lose_funds
  puts project.funding
  5.times { project.add_funds }
  puts project.funding
  puts project
  puts project.funds_needed
  project.receive_pledge
  puts project
end
